pub struct Timer { /* private fields */ }Expand description
A monotonic clock/timer that can be used to keep track of the time increments (delta time) between tick/tree traversals and the total duration since the behavior tree was first invoked/traversed
Implementations