pub enum Behavior<A> {
Show 14 variants
Wait(Float),
WaitForever,
Action(A),
Invert(Box<Behavior<A>>),
AlwaysSucceed(Box<Behavior<A>>),
Select(Vec<Behavior<A>>),
If(Box<Behavior<A>>, Box<Behavior<A>>, Box<Behavior<A>>),
Sequence(Vec<Behavior<A>>),
While(Box<Behavior<A>>, Vec<Behavior<A>>),
WhileAll(Box<Behavior<A>>, Vec<Behavior<A>>),
WhenAll(Vec<Behavior<A>>),
WhenAny(Vec<Behavior<A>>),
After(Vec<Behavior<A>>),
Race(Vec<Behavior<A>>),
// some variants omitted
}Expand description
Describes a behavior.
This is used for more complex event logic. Can also be used for game AI.
Variants
Wait(Float)
Waits an amount of time before continuing
Float: Time in seconds
WaitForever
Wait forever.
Action(A)
A high level description of an action.
An Action can either be “condition” which does not
alter the system and returns either Success or Failure
- e.g IsDoorOpen? IsNetworkDown?
Or it can be an “act” that can alter the system
and returns either Success, Failure or Running
- e.g OpenDoor, NetworkShutdown
Invert(Box<Behavior<A>>)
Converts Success into Failure and vice versa.
AlwaysSucceed(Box<Behavior<A>>)
Ignores failures and returns Success.
Select(Vec<Behavior<A>>)
Runs behaviors one by one until one succeeds.
Tries the next behavior if one fails. Fails if the last fails. A short-circuited logical OR gate.
Resumes the running child across ticks. Use .memory(false) to restart
from the first child every tick instead.
If(Box<Behavior<A>>, Box<Behavior<A>>, Box<Behavior<A>>)
If(condition, success, failure)
Sequence(Vec<Behavior<A>>)
Runs behaviors one by one until all succeed.
Fails if any behavior fails. Succeeds if all succeed. A short-circuited logical AND gate.
Resumes the running child across ticks. Use .memory(false) to restart
from the first child every tick instead.
While(Box<Behavior<A>>, Vec<Behavior<A>>)
Loops while conditional behavior is running.
Succeeds if the conditional behavior succeeds. Fails if the conditional behavior fails, or if any behavior in the loop body fails.
Panics
Panics if the given behavior sequence is empty.
WhileAll(Box<Behavior<A>>, Vec<Behavior<A>>)
Runs a sequence on repeat as long as a conditional behavior that precedes the sequence is running.
Conditional behavior is only checked before the sequence runs and not during the sequence.
Succeeds if the conditional behavior succeeds. Fails if the conditional behavior fails, or if any behavior in the sequence fails.
Panics
Panics if the given behavior sequence is empty.
use bonsai_bt::{BT, Running, Failure, Success, Action, UpdateArgs, Behavior::WhileAll, ActionArgs};
use bonsai_bt::Event;
#[derive(Clone, Debug)]
enum Ex { A, B, C }
let rs = WhileAll(
Box::new(Action(Ex::A)),
vec![Action(Ex::B), Action(Ex::C)],
);
let (SUCCESS, FAILURE, RUNNING ) = ((Success, 0.0), (Failure, 0.0), (Running, 0.0));
let mut bt = BT::new(rs, ());
let mut i = 0;
let status = bt.tick(&Event::zero_dt_args(), &mut |args: ActionArgs<Event, Ex>, _| {
match args.action {
Ex::A => {
i += 1;
if i == 4 {
SUCCESS
}
else {
RUNNING
}
}
Ex::B => {
i += 1;
SUCCESS
}
Ex::C => {
i += 1;
SUCCESS
}
}
});
assert!(i == 4);WhenAll(Vec<Behavior<A>>)
Runs all behaviors in parallel until all succeeded.
Succeeds if all behaviors succeed. Fails is any behavior fails.
WhenAny(Vec<Behavior<A>>)
Runs all behaviors in parallel until one succeeds.
Succeeds if one behavior succeeds. Fails if all behaviors failed.
After(Vec<Behavior<A>>)
Runs all behaviors in parallel until all succeeds in sequence.
Succeeds if all behaviors succeed, but only if succeeding in sequence. Fails if one behavior fails.
Race(Vec<Behavior<A>>)
Runs all behaviors in parallel until one completes (succeeds or fails).
Returns the status of the first behavior to complete,
whether that is Success or Failure.
If all behaviors remain Running, returns Running.